import itertools
import json
import logging
import os
import uuid

import pytest
from hamcrest import assert_that, contains_string, equal_to
from proton.reactor import Container

from pubtools._quay.manifest_claims_handler import (
    ManifestClaimsHandler,
    MessageHandlerTimeoutException,
    AMQPEndpointError,
    _ManifestClaimsRunner,
    UMBSettings,
)

try:
    from unittest.mock import Mock, patch, MagicMock
except ImportError:
    from mock import Mock, patch, MagicMock

DEFAULT_AMQPS_URL = "amqps://{hostname}:{port}".format(hostname="localhost", port=5672)

HERE = os.path.dirname(__file__)
UMB_CERT_DIR = os.path.normpath(os.path.join(HERE, "../", "integration", "utils", "umb_certs"))

ROOT_CA = os.path.join(UMB_CERT_DIR, "rootCA.pem")

PUB_UMB_CLIENT_CERT = os.path.join(UMB_CERT_DIR, "pub-cert-key.pem")

TEST_TOPIC = "tests.manifestclaimshandler.TestTopic"

# flake8: noqa: D101, D107, D102, D205, D400, D403


@pytest.fixture
def debug_logs(caplog):
    caplog.set_level(logging.DEBUG)
    return caplog


@pytest.fixture
def umb_client_settings():
    return UMBSettings(
        [DEFAULT_AMQPS_URL],
        radas_address=TEST_TOPIC,
        pub_cert=PUB_UMB_CLIENT_CERT,
        ca_cert=ROOT_CA,
        signing_timeout=5,
        signing_throttle=10,
        signing_retry=3,
    )


@pytest.fixture
def handler(umb_client_settings):
    """Yields a ManifestClaimsHandler instance with no messages."""
    with patch("proton.SSLDomain"):
        yield ManifestClaimsHandler(umb_client_settings, [], message_sender_callback=lambda: None)


@pytest.fixture
def runner(umb_client_settings):
    return _ManifestClaimsRunner(umb_client_settings, [], lambda: None)


@pytest.mark.parametrize(
    "event_name",
    [
        "connection_closing",
        "connection_closed",
        "connection_error",
        "session_closing",
        "session_closed",
        "session_error",
        "link_closing",
        "link_closed",
        "link_error",
        "transport_error",
        "disconnected",
    ],
)
def test_logs_event(handler, event_name, debug_logs):
    """ManifestClaimsHandler logs a message when various events are received."""
    event = Mock()
    handler._endpoint_error = MagicMock()

    callback = getattr(handler, "on_%s" % event_name)
    # It should execute successfully.
    callback(event)

    logs = debug_logs.text

    # It should log the name of the event which occurred.
    assert_that(logs, contains_string("Messaging event: {0}".format(event_name)))


def test_logs_unexpected_link_opened(handler, caplog):
    """ManifestClaimsHandler logs a message when on_link_opened
    event is received with an unexpected receiver."""

    event = Mock(receiver=object())

    # It should execute successfully.
    handler.on_link_opened(event)

    logs = caplog.text

    # It should warn due to event with unexpected receiver.
    assert "Unexpected on_link_opened event" in logs


def test_logs_ignored_message(handler, debug_logs):
    """ManifestClaimsHandler logs a message when receiving a radas message
    which can't be linked to a request within the current task."""
    message_body = json.dumps({"msg": {"request_id": "abcdef"}})
    event = Mock(message=Mock(body=message_body))

    # It should execute successfully.
    handler.on_message(event)

    logs = debug_logs.text

    # It should mention that we got a message, but it's not relevant
    # to our handler.
    assert "Ignored signing response: abcdef" in logs


@patch("monotonic.monotonic")
def test_awaiting_response_time_out(fake_timer, handler, caplog):
    handler.timeout = 10
    handler.connected = True
    handler.awaiting_response = {"request_id1": 43190}
    handler.retry_count = {"request_id1": 3}
    # assume it's been tried three times
    fake_timer.return_value = 43210

    event = MagicMock(container=MagicMock(spec=Container))

    with pytest.raises(MessageHandlerTimeoutException):
        handler.on_timer_task(event)

    event.container.stop.assert_called_once()

    logs = caplog.text

    assert "Stopping message event loop due to timeout request_id1" in logs


@patch("monotonic.monotonic")
def test_awaiting_response_retry(fake_timer, handler, caplog):
    handler.timeout = 10
    handler.throttle = 5
    handler.awaiting_response = {"request_id1": 43190}
    handler.retry_count = {"request_id1": 2}
    handler.id_msg_map = {"request_id1": {"request_id": "request_id1", "msg": "message1"}}
    handler.to_send = []
    handler.connected = True
    handler.message_sender_callback = Mock()
    for i in range(2, 11):
        handler.to_send.append({"request_id": "request_id%s" % i, "msg": "message%s" % i})
    # assume it's been tried twice
    fake_timer.side_effect = [43210 + i for i in range(6)]

    event = Mock()

    handler.on_timer_task(event)

    # 5 more messages in queue should be sent
    assert len(handler.awaiting_response) == 5
    # there should be 4+1 message left
    assert len(handler.to_send) == 5
    # the last one is the one timeouted
    assert handler.to_send[-1] == {"request_id": "request_id1", "msg": "message1"}

    assert event.container.schedule.called

    logs = caplog.text
    assert "Didn't receive response in 10 for request request_id1, will retry [3/3]" in logs


def test_cannot_connect_umb_timeout(handler, caplog):
    event = Mock()

    with pytest.raises(MessageHandlerTimeoutException):
        handler.on_timer_task(event)

    logs = caplog.text

    assert "Couldn't connect to brokers after {0} seconds".format(handler.timeout) in logs


def test_non_fatal_transport_error(handler):
    condition = MagicMock(name="Fake Name", description="Fake Description")
    endpoint = MagicMock(name="Transport", condition=condition)
    event = MagicMock(transport=endpoint)

    handler._endpoint_error_if_unhandled = MagicMock()
    handler._on_error = MagicMock()
    handler.on_transport_error(event)
    handler._endpoint_error_if_unhandled.assert_called_once_with(event, endpoint)
    handler._on_error.assert_not_called()


def test_fatal_transport_error(handler):
    condition = MagicMock(description="local-idle-timeout expired")
    # Passing name as a keyword argument to MagicMock constructor
    # conflicts with the MagicMock#name field, and produces something
    # like <MagicMock name="amqp:resource-limit-exceeded" id=<some integer>
    # when name is accessed again. Overwrite name entirely here.
    condition.name = "amqp:resource-limit-exceeded"
    endpoint = MagicMock(name="Transport", condition=condition)
    event = MagicMock(transport=endpoint)

    handler._on_error = MagicMock()
    handler.on_transport_error(event)
    handler._on_error.assert_called_once()


@pytest.mark.parametrize(
    "endpoint_name,condition",
    itertools.product(("connection", "session", "link"), (("FakeName", "FakeDescription"), None)),
)
def test_other_endpoint_errors(endpoint_name, condition, handler, debug_logs):
    if condition is not None:
        name, description = condition
        condition = MagicMock(description=description)
        setattr(condition, "name", name)

    endpoint = MagicMock(name=endpoint_name, condition=condition, remote_condition=condition)
    endpoint.__class__.__name__ = endpoint_name.capitalize()

    event = MagicMock()
    setattr(event, endpoint_name, endpoint)

    handler._on_error = MagicMock()
    method = getattr(handler, "on_{endpoint}_error".format(endpoint=endpoint_name))
    method(event)
    handler._on_error.assert_called_once()

    logs = debug_logs.text
    label = condition.name if condition is not None else "Unknown"
    if condition is not None:
        description = condition.description
    else:
        description = "No error description provided."

    expected_error = AMQPEndpointError(
        endpoint_name=endpoint_name.capitalize(),
        error_label=label,
        description=description,
    )
    assert_that(logs, contains_string(str(expected_error)))


def test_manifest_claims_runner_on_error_restarts(runner, debug_logs):
    claims = [{"request_id": uuid.uuid4(), "msg": {"field": "value"}}]

    uid = uuid.uuid4()
    received = {uid: {"request_id": uid, "msg": {"field": "value"}}}

    runner._claim_messages = claims
    runner._received_messages = received
    runner._run = MagicMock()
    runtime_error_message = "Runtime error message"
    exception = RuntimeError(runtime_error_message)

    runner.on_error(exception)

    assert_that(runner._retry_attempts, equal_to(1))
    runner._run.assert_called_once_with(claims)
    logs = debug_logs.text
    check_strings = [
        "Error in message handler: {0}".format(exception),
        "Restarting message handler for {0} remaining claims (retry {1}/{2})".format(
            len(claims), runner._retry_attempts, runner._maximum_retries
        ),
    ]
    for string in check_strings:
        assert_that(logs, contains_string(string))


def test_manifest_claims_runner_on_error_no_restart(runner, debug_logs):
    claims = [{"request_id": uuid.uuid4(), "msg": {"field": "value"}}]

    received = dict((m["request_id"], m) for m in claims)

    runner._claim_messages = claims
    runner._received_messages = received
    runner._run = MagicMock()

    runtime_error_message = "Runtime error message"
    exception = RuntimeError(runtime_error_message)

    runner.on_error(exception)

    assert_that(runner._retry_attempts, equal_to(0))
    runner._run.assert_not_called()

    logs = debug_logs.text
    check_strings = [
        "Error in message handler: {0}".format(exception),
        "No claims missing responses. Restart not required.",
    ]
    for string in check_strings:
        assert_that(logs, contains_string(string))


def test_manifest_claims_runner_no_more_retries(runner, debug_logs):
    runner._run = MagicMock()
    runner._retry_attempts = runner._maximum_retries + 1

    runtime_error_message = "Runtime error message"
    exception = RuntimeError(runtime_error_message)

    with pytest.raises(RuntimeError, match="Retry limit reached\. Message handler has failed .*"):
        runner.on_error(exception)

    runner._run.assert_not_called()

    logs = debug_logs.text
    check_strings = [
        "Error in message handler: {0}".format(exception),
    ]
    for string in check_strings:
        assert_that(logs, contains_string(string))



import mock
import pytest

from pubtools._quay import exceptions
from pubtools._quay import iib_operations
from .utils.misc import IIBRes, mock_manifest_list_requests
from .fake_quay_client import FakeQuayClient

import requests_mock

# flake8: noqa: E501

FAKE_MANIFEST_LIST = {
    "manifests": [
        {
            "mediaType": "application/vnd.docker.distribution.manifest.v2+json",
            "size": 7143,
            "digest": "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
            "platform": {
                "architecture": "amd64",
                "os": "linux",
            },
        },
        {
            "mediaType": "application/vnd.docker.distribution.manifest.v2+json",
            "size": 7143,
            "digest": "sha256:4c8a0e4802b39cad2608a367c6361b5b3cbedf18d8432922e253b80715be94c1",
            "platform": {
                "architecture": "ppc64le",
                "os": "linux",
            },
        },
        {
            "mediaType": "application/vnd.docker.distribution.manifest.v2+json",
            "size": 7143,
            "digest": "sha256:20e5e8823781e9de37b20940719f8c803183399db0d91fe1d22fce9b43acfb7f",
            "platform": {
                "architecture": "s390x",
                "os": "linux",
            },
        },
    ]
}

MSG_SIGNER_OPERATION_RESULT = [
    [
        {
            "i": 0,
            "msg": {
                "errors": [],
                "manifest_digest": "sha256:bd6eba96070efe86b64"
                "b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                "pub_task_id": "12345",
                "repo": "iib-namespace/new-index-image",
                "request_id": "89cf86e0-8403-46e0-b5ed-5984a635e89e",
                "request_received_time": "2023-10-17T08:08:01.544757",
                "sig_key_id": "37036783",
                "sig_keyname": "testing",
                "signature_type": "container_signature",
                "signed_claim": "claim1",
            },
        },
        {},
    ]
]


@pytest.fixture
def mock_timestamp():
    with mock.patch("pubtools._quay.iib_operations.timestamp") as mocked:
        mocked.return_value = "timestamp"
        yield mocked


@pytest.fixture
def fake_quay_client():
    return FakeQuayClient()


@pytest.fixture
def fake_quay_client_get_operator_quay_client(fake_quay_client):
    with mock.patch("pubtools._quay.iib_operations._get_operator_quay_client") as mocked:
        mocked.return_value = fake_quay_client
        yield fake_quay_client


@pytest.fixture
def msg_signer_wrapper_save_signatures_file():
    with mock.patch(
        "pubtools._quay.signer_wrapper.MsgSignerWrapper._save_signatures_file"
    ) as mocked:
        mocked.return_value.__enter__.return_value.name = "signature_file"
        yield mocked


def test_verify_target_settings_success(target_settings):
    iib_operations.verify_target_settings(target_settings)


def test_verify_target_settings_missing_setting(target_settings):
    target_settings.pop("dest_quay_user")
    with pytest.raises(
        exceptions.InvalidTargetSettings, match="'dest_quay_user' must be present.*"
    ):
        iib_operations.verify_target_settings(target_settings)


def test_verify_target_settings_missing_docker_setting(target_settings):
    target_settings["docker_settings"].pop("umb_urls")
    with pytest.raises(
        exceptions.InvalidTargetSettings,
        match="'umb_urls' must be present in the docker settings.*",
    ):
        iib_operations.verify_target_settings(target_settings)


@pytest.fixture
def mock_get_index_image_signatures():
    mock_get_index_image_signatures = mock.MagicMock()
    mock_get_index_image_signatures.return_value = [
        {
            "signature": "value1",
            "_id": "1",
            "reference": "some-registry.com/redhat-namespace/old-index-image:8",
            "repository": "image-repo",
            "manifest_digest": "sha256:a1a1a",
            "sig_key_id": "sig-key",
        },
        {
            "signature": "value2",
            "_id": "2",
            "reference": "some-registry2.com/redhat-namespace/old-index-image:8",
            "repository": "image-repo",
            "manifest_digest": "sha256:b2b2b2",
            "sig_key_id": "sig-key",
        },
    ]
    return mock_get_index_image_signatures


def fake_setup(
    fake_quay_client_get_operator_quay_client,
    mock_iib_add_bundles,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
):
    fake_quay_client_get_operator_quay_client.f_add_manifest(
        "some-registry.com/iib-namespace/iib@sha256:a1a1a1",
        FAKE_MANIFEST_LIST,
        FakeQuayClient.MANIFEST_LIST_TYPE,
        "sha256:a1a1a1",
    )
    build_details = IIBRes(
        "some-registry.com/iib-namespace/new-index-image:8",
        "some-registry.com/iib-namespace/iib@sha256:a1a1a1",
        ["8-1"],
    )
    signer_wrapper_run_entry_point_sf = [
        # get signatures from pyxis
        [
            {
                "_id": 1,
                "manifest_digest": "sha256:5555555555",
                "reference": "some-registry.com/operators/index-image:8",
                "sig_key_id": "key",
                "repository": "operators/index-image",
            }
        ],
        [
            {
                "_id": 1,
                "manifest_digest": "sha256:5555555555",
                "reference": "some-registry.com/operators/index-image:8-timestamp",
                "sig_key_id": "key",
                "repository": "operators/index-image",
            }
        ],
        # filter existing
        [
            {
                "_id": 1,
                "manifest_digest": "sha256:5555555555",
                "reference": "some-registry.com/operators/index-image:8",
                "sig_key_id": "key",
                "repository": "operators/index-image",
            }
        ],
        (True, ["some-registry.com/operators/index-image:8.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["some-registry.com/operators/index-image:8-timestamp.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        (True, ["quay.io/testing/repo:sha256-5555555555.sig"]),
        # store signatures to pyxis
        [
            {
                "_id": 1,
                "manifest_digest": "sha256:5555555555",
                "reference": "some-registry.com/operators/index-image:8",
                "sig_key_id": "key",
                "repository": "operators/index-image",
            }
        ],
        [
            {
                "_id": 1,
                "manifest_digest": "sha256:5555555555",
                "reference": "some-registry.com/operators/index-image:8-timestamp",
                "sig_key_id": "key",
                "repository": "operators/index-image",
            }
        ],
    ]
    signer_wrapper_run_entry_point_sf.append(
        # remove signatures from pyxis (fetch existing)
        [
            {
                "_id": 1,
                "manifest_digest": "sha256:5555555555",
                "reference": "some-registry.com/operators/index-image:8",
                "sig_key_id": "key",
                "repository": "operators/index-image",
            }
        ]
    )
    signer_wrapper_run_entry_point_sf.append((True, ["quay.io/testing/repo:sha256-5555555555.sig"]))
    signer_wrapper_run_entry_point_sf.append((True, ["quay.io/testing/repo:sha256-5555555555.sig"]))

    signer_wrapper_run_entry_point.side_effect = signer_wrapper_run_entry_point_sf

    mock_iib_add_bundles.return_value = build_details
    signer_wrapper_entry_point.return_value = {
        "signer_result": {
            "status": "ok",
        },
        "operation": {
            "references": ["some-registry.com/iib-namespace/new-index-image:8"],
            "manifests": [
                "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6"
            ],
        },
        "operation_results": MSG_SIGNER_OPERATION_RESULT,
        "signing_key": "sig-key",
    }


@mock.patch("pubtools._quay.iib_operations.ContainerImagePusher.run_tag_images")
@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_add_bundles")
@mock.patch("pubtools._quay.iib_operations.verify_target_settings")
def test_task_iib_add_bundles(
    mock_verify_target_settings,
    mock_iib_add_bundles,
    mock_run_tag_images,
    fake_quay_client_get_operator_quay_client,
    mock_timestamp,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
    signer_wrapper_remove_signatures,
    msg_signer_wrapper_save_signatures_file,
    target_settings,
    fake_cert_key_paths,
    v2s1_manifest,
    src_manifest_list,
):
    fake_setup(
        fake_quay_client_get_operator_quay_client,
        mock_iib_add_bundles,
        signer_wrapper_entry_point,
        signer_wrapper_run_entry_point,
    )
    with requests_mock.Mocker() as m:
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/some-namespace/operators----index-image/manifests/8",
            src_manifest_list,
            v2s1_manifest,
        )
        # manifests for removal of old signatures
        m.get(
            "https://quay.io/v2/"
            "some-namespace/operators----index-image/manifests/manifest_list_digest",
            json=src_manifest_list,
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json",
                "docker-content-digest": "manifest_list_digest",
            },
        )
        iib_operations.task_iib_add_bundles(
            ["bundle1", "bundle2"],
            ["arch1", "arch2"],
            "some-registry.com/redhat-namespace/new-index-image:8",
            ["bundle3", "bundle4"],
            ["some-key"],
            "1",
            target_settings,
        )

    mock_verify_target_settings.assert_called_once_with(target_settings)
    mock_iib_add_bundles.assert_called_once_with(
        bundles=["bundle1", "bundle2"],
        archs=["arch1", "arch2"],
        index_image="some-registry.com/redhat-namespace/new-index-image:8",
        deprecation_list=["bundle3", "bundle4"],
        build_tags=["8-1"],
        target_settings=target_settings,
    )
    signer_wrapper_entry_point.assert_has_calls(
        [
            mock.call(
                config_file="test-config.yml",
                signing_key="some-key",
                reference=[
                    "some-registry1.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8-timestamp",
                ],
                digest=[
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                ],
                task_id="1",
            ),
            # cosign
            mock.call(
                config_file="test-config.yml",
                signing_key="some-key",
                reference=[
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                ],
                digest=[
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                ],
                identity=[
                    "some-registry1.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8-timestamp",
                    "some-registry1.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8-timestamp",
                ],
            ),
        ]
    )
    assert mock_run_tag_images.call_count == 2
    assert mock_run_tag_images.call_args_list[0] == mock.call(
        "some-registry.com/iib-namespace/new-index-image:8",
        [
            "quay.io/some-namespace/operators----index-image:8",
        ],
        True,
        target_settings,
    )
    assert mock_run_tag_images.call_args_list[1] == mock.call(
        "some-registry.com/iib-namespace/iib:8-1",
        [
            "quay.io/some-namespace/operators----index-image:8-timestamp",
        ],
        True,
        target_settings,
    )
    assert signer_wrapper_remove_signatures.mock_calls == [
        mock.call([1]),
        # TODO: Uncomment when cosign removing signatures is enabled
        # mock.call([("operators/index-image", "sha256:5555555555")]),
    ]


@mock.patch("pubtools._quay.iib_operations.ContainerImagePusher.run_tag_images")
@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_add_bundles")
@mock.patch("pubtools._quay.iib_operations.verify_target_settings")
def test_task_iib_add_bundles_missing_manifest_list(
    mock_verify_target_settings,
    mock_iib_add_bundles,
    mock_run_tag_images,
    fake_quay_client_get_operator_quay_client,
    mock_timestamp,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
    signer_wrapper_remove_signatures,
    msg_signer_wrapper_save_signatures_file,
    target_settings,
    fake_cert_key_paths,
    v2s1_manifest,
    src_manifest_list,
):
    fake_setup(
        fake_quay_client_get_operator_quay_client,
        mock_iib_add_bundles,
        signer_wrapper_entry_point,
        signer_wrapper_run_entry_point,
    )
    with requests_mock.Mocker() as m:
        m.get(
            "https://quay.io/v2/some-namespace/operators----index-image/manifests/8",
            json=None,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        iib_operations.task_iib_add_bundles(
            ["bundle1", "bundle2"],
            ["arch1", "arch2"],
            "some-registry.com/redhat-namespace/new-index-image:8",
            ["bundle3", "bundle4"],
            ["some-key"],
            "1",
            target_settings,
        )

    mock_verify_target_settings.assert_called_once_with(target_settings)
    mock_iib_add_bundles.assert_called_once_with(
        bundles=["bundle1", "bundle2"],
        archs=["arch1", "arch2"],
        index_image="some-registry.com/redhat-namespace/new-index-image:8",
        deprecation_list=["bundle3", "bundle4"],
        build_tags=["8-1"],
        target_settings=target_settings,
    )
    signer_wrapper_entry_point.assert_has_calls(
        [
            mock.call(
                config_file="test-config.yml",
                signing_key="some-key",
                reference=[
                    "some-registry1.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8-timestamp",
                ],
                digest=[
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                ],
                task_id="1",
            ),
            # cosign
            mock.call(
                config_file="test-config.yml",
                signing_key="some-key",
                reference=[
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                ],
                digest=[
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                ],
                identity=[
                    "some-registry1.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8-timestamp",
                    "some-registry1.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8-timestamp",
                ],
            ),
        ]
    )
    assert mock_run_tag_images.call_count == 2
    assert mock_run_tag_images.call_args_list[0] == mock.call(
        "some-registry.com/iib-namespace/new-index-image:8",
        [
            "quay.io/some-namespace/operators----index-image:8",
        ],
        True,
        target_settings,
    )
    assert mock_run_tag_images.call_args_list[1] == mock.call(
        "some-registry.com/iib-namespace/iib:8-1",
        [
            "quay.io/some-namespace/operators----index-image:8-timestamp",
        ],
        True,
        target_settings,
    )
    assert signer_wrapper_remove_signatures.mock_calls == [
        # TODO: Uncomment when cosign removing signatures is enabled
        # mock.call([]),
        mock.call([]),
    ]


@mock.patch("pubtools._quay.iib_operations.ContainerImagePusher.run_tag_images")
@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_add_bundles")
@mock.patch("pubtools._quay.iib_operations.verify_target_settings")
def test_task_iib_add_bundles_operator_ns(
    mock_verify_target_settings,
    mock_iib_add_bundles,
    mock_run_tag_images,
    mock_timestamp,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
    msg_signer_wrapper_save_signatures_file,
    signer_wrapper_remove_signatures,
    fake_quay_client_get_operator_quay_client,
    target_settings,
    fake_cert_key_paths,
    src_manifest_list,
    v2s1_manifest,
    fixture_run_in_parallel_signer,
):
    target_settings["quay_operator_namespace"] = "operator-ns"

    fake_setup(
        fake_quay_client_get_operator_quay_client,
        mock_iib_add_bundles,
        signer_wrapper_entry_point,
        signer_wrapper_run_entry_point,
    )
    with requests_mock.Mocker() as m:
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/operator-ns/operators----index-image/manifests/8",
            src_manifest_list,
            v2s1_manifest,
        )
        # call to remove old signatures
        m.get(
            "https://quay.io/"
            "v2/some-namespace/operators----index-image/manifests/manifest_list_digest",
            json=src_manifest_list,
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json",
                "docker-content-digest": "manifest_list_digest",
            },
        )
        m.get(
            "https://quay.io"
            "/v2/some-namespace/operators----index-image/manifests/sha256:1111111111",
            json=v2s1_manifest,
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.v2+json",
                "docker-content-digest": "manifest_list_digest",
            },
        )
        m.get(
            "https://quay.io"
            "/v2/some-namespace/operators----index-image/manifests/sha256:2222222222",
            json=v2s1_manifest,
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.v2+json",
                "docker-content-digest": "manifest_list_digest",
            },
        )
        m.get(
            "https://quay.io"
            "/v2/some-namespace/operators----index-image/manifests/sha256:3333333333",
            json=v2s1_manifest,
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.v2+json",
                "docker-content-digest": "manifest_list_digest",
            },
        )
        m.get(
            "https://quay.io"
            "/v2/some-namespace/operators----index-image/manifests/sha256:5555555555",
            json=v2s1_manifest,
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.v2+json",
                "docker-content-digest": "manifest_list_digest",
            },
        )
        iib_operations.task_iib_add_bundles(
            ["bundle1", "bundle2"],
            ["arch1", "arch2"],
            "some-registry.com/redhat-namespace/new-index-image:8",
            ["bundle3", "bundle4"],
            ["some-key"],
            "1",
            target_settings,
        )

    mock_verify_target_settings.assert_called_once_with(target_settings)
    mock_iib_add_bundles.assert_called_once_with(
        bundles=["bundle1", "bundle2"],
        archs=["arch1", "arch2"],
        index_image="some-registry.com/redhat-namespace/new-index-image:8",
        deprecation_list=["bundle3", "bundle4"],
        build_tags=["8-1"],
        target_settings=target_settings,
    )
    signer_wrapper_entry_point.assert_has_calls(
        [
            mock.call(
                config_file="test-config.yml",
                signing_key="some-key",
                reference=[
                    "some-registry1.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8-timestamp",
                ],
                digest=[
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                ],
                task_id="1",
            ),
            # cosign
            mock.call(
                config_file="test-config.yml",
                signing_key="some-key",
                reference=[
                    "quay.io/operator-ns/operators----index-image:8",
                    "quay.io/operator-ns/operators----index-image:8",
                    "quay.io/operator-ns/operators----index-image:8-timestamp",
                    "quay.io/operator-ns/operators----index-image:8-timestamp",
                    "quay.io/operator-ns/operators----index-image:8",
                    "quay.io/operator-ns/operators----index-image:8-timestamp",
                    "quay.io/operator-ns/operators----index-image:8",
                    "quay.io/operator-ns/operators----index-image:8-timestamp",
                ],
                digest=[
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                ],
                identity=[
                    "some-registry1.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8-timestamp",
                    "some-registry1.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8-timestamp",
                ],
            ),
        ]
    )
    signer_wrapper_run_entry_point.assert_has_calls(
        [
            mock.call(
                ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-get-signatures"),
                "pubtools-pyxis-get-signatures",
                [
                    "--pyxis-server",
                    "pyxis-url.com",
                    "--pyxis-ssl-crtfile",
                    "/path/to/file.crt",
                    "--pyxis-ssl-keyfile",
                    "/path/to/file.key",
                    "--manifest-digest",
                    mock.ANY,
                ],
                {},
            ),
            mock.call(
                ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-upload-signatures"),
                "pubtools-pyxis-upload-signature",
                [
                    "--pyxis-server",
                    "pyxis-url.com",
                    "--pyxis-ssl-crtfile",
                    "/path/to/file.crt",
                    "--pyxis-ssl-keyfile",
                    "/path/to/file.key",
                    "--request-threads",
                    "7",
                    "--signatures",
                    mock.ANY,
                ],
                {},
                False,
            ),
            mock.call(
                ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-get-signatures"),
                "pubtools-pyxis-get-signatures",
                [
                    "--pyxis-server",
                    "pyxis-url.com",
                    "--pyxis-ssl-crtfile",
                    "/path/to/file.crt",
                    "--pyxis-ssl-keyfile",
                    "/path/to/file.key",
                    "--manifest-digest",
                    mock.ANY,
                ],
                {},
            ),
            # Uncomment when cosign signature removal is enabled
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@manifest_list_digest",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:1111111111",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:2222222222",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:3333333333",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:5555555555",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:1111111111",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:2222222222",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:3333333333",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:5555555555",
            #     ],
            #     {},
            # ),
        ]
    )
    msg_signer_wrapper_save_signatures_file.assert_any_call(
        [
            {
                "manifest_digest": "sha256:bd6eba96070efe86b64b9"
                "a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                "reference": "some-registry.com/iib-namespace/new-index-image:8",
                "repository": "iib-namespace/new-index-image",
                "sig_key_id": "sig-key",
                "signature_data": "claim1",
            }
        ]
    )

    assert mock_run_tag_images.call_count == 2
    assert mock_run_tag_images.call_args_list[0] == mock.call(
        "some-registry.com/iib-namespace/new-index-image:8",
        [
            "quay.io/operator-ns/operators----index-image:8",
        ],
        True,
        target_settings,
    )
    assert mock_run_tag_images.call_args_list[1] == mock.call(
        "some-registry.com/iib-namespace/iib:8-1",
        [
            "quay.io/operator-ns/operators----index-image:8-timestamp",
        ],
        True,
        target_settings,
    )
    assert signer_wrapper_remove_signatures.mock_calls == [
        mock.call([1]),
        # TODO: Uncomment when cosign removing signatures is enabled
        # mock.call([("operators/index-image", "sha256:5555555555")]),
    ]


@mock.patch("pubtools._quay.iib_operations.ContainerImagePusher.run_tag_images")
@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_remove_operators")
@mock.patch("pubtools._quay.iib_operations.verify_target_settings")
def test_task_iib_remove_operators(
    mock_verify_target_settings,
    mock_iib_remove_operators,
    mock_run_tag_images,
    mock_timestamp,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
    msg_signer_wrapper_save_signatures_file,
    signer_wrapper_remove_signatures,
    fake_quay_client_get_operator_quay_client,
    target_settings,
    fake_cert_key_paths,
    v2s1_manifest,
    src_manifest_list,
    fixture_run_in_parallel,
):
    fake_setup(
        fake_quay_client_get_operator_quay_client,
        mock_iib_remove_operators,
        signer_wrapper_entry_point,
        signer_wrapper_run_entry_point,
    )

    with requests_mock.Mocker() as m:
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/some-namespace/operators----index-image/manifests/8",
            src_manifest_list,
            v2s1_manifest,
        )
        # manifests for removal of old signatures
        m.get(
            "https://quay.io"
            "/v2/some-namespace/operators----index-image/manifests/manifest_list_digest",
            json=src_manifest_list,
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json",
                "docker-content-digest": "manifest_list_digest",
            },
        )

        iib_operations.task_iib_remove_operators(
            ["operator1", "operator2"],
            ["arch1", "arch2"],
            "some-registry.com/redhat-namespace/new-index-image:5",
            ["some-key"],
            "1",
            target_settings,
        )

    mock_verify_target_settings.assert_called_once_with(target_settings)
    mock_iib_remove_operators.assert_called_once_with(
        operators=["operator1", "operator2"],
        archs=["arch1", "arch2"],
        index_image="some-registry.com/redhat-namespace/new-index-image:5",
        build_tags=["5-1"],
        target_settings=target_settings,
    )
    assert mock_run_tag_images.call_count == 2
    assert mock_run_tag_images.call_args_list[0] == mock.call(
        "some-registry.com/iib-namespace/new-index-image:8",
        [
            "quay.io/some-namespace/operators----index-image:8",
        ],
        True,
        target_settings,
    )
    assert mock_run_tag_images.call_args_list[1] == mock.call(
        "some-registry.com/iib-namespace/iib:8-1",
        [
            "quay.io/some-namespace/operators----index-image:8-timestamp",
        ],
        True,
        target_settings,
    )
    signer_wrapper_remove_signatures.mock_calls == [
        mock.call([1]),
        mock.call([("operators/index-image", "sha256:5555555555")]),
    ]


@mock.patch("pubtools._quay.iib_operations.ContainerImagePusher.run_tag_images")
@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_remove_operators")
@mock.patch("pubtools._quay.iib_operations.verify_target_settings")
def test_task_iib_remove_operators_missing_manifest_list(
    mock_verify_target_settings,
    mock_iib_remove_operators,
    mock_run_tag_images,
    mock_timestamp,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
    msg_signer_wrapper_save_signatures_file,
    signer_wrapper_remove_signatures,
    fake_quay_client_get_operator_quay_client,
    target_settings,
    fake_cert_key_paths,
    v2s1_manifest,
    src_manifest_list,
    fixture_run_in_parallel_signer,
):
    fake_setup(
        fake_quay_client_get_operator_quay_client,
        mock_iib_remove_operators,
        signer_wrapper_entry_point,
        signer_wrapper_run_entry_point,
    )

    with requests_mock.Mocker() as m:
        m.get(
            "https://quay.io/v2/some-namespace/operators----index-image/manifests/8",
            json=None,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        iib_operations.task_iib_remove_operators(
            ["operator1", "operator2"],
            ["arch1", "arch2"],
            "some-registry.com/iib-namespace/new-index-image:8",
            ["some-key"],
            "1",
            target_settings,
        )

    mock_verify_target_settings.assert_called_once_with(target_settings)
    mock_iib_remove_operators.assert_called_once_with(
        operators=["operator1", "operator2"],
        archs=["arch1", "arch2"],
        index_image="some-registry.com/iib-namespace/new-index-image:8",
        build_tags=["8-1"],
        target_settings=target_settings,
    )
    assert mock_run_tag_images.call_count == 2
    assert mock_run_tag_images.call_args_list[0] == mock.call(
        "some-registry.com/iib-namespace/new-index-image:8",
        [
            "quay.io/some-namespace/operators----index-image:8",
        ],
        True,
        target_settings,
    )
    assert mock_run_tag_images.call_args_list[1] == mock.call(
        "some-registry.com/iib-namespace/iib:8-1",
        [
            "quay.io/some-namespace/operators----index-image:8-timestamp",
        ],
        True,
        target_settings,
    )
    signer_wrapper_remove_signatures.mock_calls == [
        mock.call([1]),
        mock.call([("operators/index-image", "sha256:5555555555")]),
    ]


@mock.patch("pubtools._quay.iib_operations.ContainerImagePusher.run_tag_images")
@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_remove_operators")
@mock.patch("pubtools._quay.iib_operations.verify_target_settings")
def test_task_iib_remove_operators_operator_ns(
    mock_verify_target_settings,
    mock_iib_remove_operators,
    mock_run_tag_images,
    mock_timestamp,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
    msg_signer_wrapper_save_signatures_file,
    signer_wrapper_remove_signatures,
    fake_quay_client_get_operator_quay_client,
    target_settings,
    fake_cert_key_paths,
    src_manifest_list,
    v2s1_manifest,
    fixture_run_in_parallel_signer,
):
    target_settings["quay_operator_namespace"] = "operator-ns"

    fake_setup(
        fake_quay_client_get_operator_quay_client,
        mock_iib_remove_operators,
        signer_wrapper_entry_point,
        signer_wrapper_run_entry_point,
    )
    with requests_mock.Mocker() as m:
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/operator-ns/operators----index-image/manifests/8",
            src_manifest_list,
            v2s1_manifest,
        )
        # call to remove old signatures
        mock_manifest_list_requests(
            m,
            "https://quay.io"
            "/v2/some-namespace/operators----index-image/manifests/manifest_list_digest",
            src_manifest_list,
            v2s1_manifest,
        )
        iib_operations.task_iib_remove_operators(
            ["operator1", "operator2"],
            ["arch1", "arch2"],
            "some-registry.com/redhat-namespace/new-index-image:5",
            ["some-key"],
            "1",
            target_settings,
        )

    mock_verify_target_settings.assert_called_once_with(target_settings)
    mock_iib_remove_operators.assert_called_once_with(
        operators=["operator1", "operator2"],
        archs=["arch1", "arch2"],
        index_image="some-registry.com/redhat-namespace/new-index-image:5",
        build_tags=["5-1"],
        target_settings=target_settings,
    )
    assert mock_run_tag_images.call_count == 2
    assert mock_run_tag_images.call_args_list[0] == mock.call(
        "some-registry.com/iib-namespace/new-index-image:8",
        [
            "quay.io/operator-ns/operators----index-image:8",
        ],
        True,
        target_settings,
    )
    assert mock_run_tag_images.call_args_list[1] == mock.call(
        "some-registry.com/iib-namespace/iib:8-1",
        [
            "quay.io/operator-ns/operators----index-image:8-timestamp",
        ],
        True,
        target_settings,
    )
    signer_wrapper_remove_signatures.mock_calls == [
        mock.call([1]),
        mock.call([("operators/index-image", "sha256:5555555555")]),
    ]


@mock.patch("pubtools._quay.iib_operations.ContainerImagePusher.run_tag_images")
@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_add_bundles")
@mock.patch("pubtools._quay.iib_operations.verify_target_settings")
def test_task_iib_build_from_scratch(
    mock_verify_target_settings,
    mock_iib_add_bundles,
    mock_run_tag_images,
    mock_timestamp,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
    msg_signer_wrapper_save_signatures_file,
    signer_wrapper_remove_signatures,
    fake_quay_client_get_operator_quay_client,
    target_settings,
    fake_cert_key_paths,
    src_manifest_list,
    v2s1_manifest,
    fixture_run_in_parallel_signer,
):
    fake_setup(
        fake_quay_client_get_operator_quay_client,
        mock_iib_add_bundles,
        signer_wrapper_entry_point,
        signer_wrapper_run_entry_point,
    )
    with requests_mock.Mocker() as m:
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/some-namespace/operators----index-image/manifests/8",
            src_manifest_list,
            v2s1_manifest,
        )
        # call to remove old signatures
        m.get(
            "https://quay.io/v2"
            "/some-namespace/operators----index-image/manifests/manifest_list_digest",
            json=src_manifest_list,
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json",
                "docker-content-digest": "manifest_list_digest",
            },
        )
        iib_operations.task_iib_build_from_scratch(
            ["bundle1", "bundle2"],
            ["arch1", "arch2"],
            "8",
            ["some-key"],
            "1",
            target_settings,
        )

    mock_verify_target_settings.assert_called_once_with(target_settings)
    mock_iib_add_bundles.assert_called_once_with(
        bundles=["bundle1", "bundle2"],
        archs=["arch1", "arch2"],
        build_tags=["8-1"],
        target_settings=target_settings,
    )
    signer_wrapper_entry_point.assert_has_calls(
        [
            mock.call(
                config_file="test-config.yml",
                signing_key="some-key",
                reference=[
                    "some-registry1.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8-timestamp",
                ],
                digest=[
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                ],
                task_id="1",
            ),
            # cosign
            mock.call(
                config_file="test-config.yml",
                signing_key="some-key",
                reference=[
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                ],
                digest=[
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                ],
                identity=[
                    "some-registry1.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8-timestamp",
                    "some-registry1.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8-timestamp",
                ],
            ),
        ]
    )
    signer_wrapper_run_entry_point.assert_has_calls(
        [
            mock.call(
                ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-get-signatures"),
                "pubtools-pyxis-get-signatures",
                [
                    "--pyxis-server",
                    "pyxis-url.com",
                    "--pyxis-ssl-crtfile",
                    "/path/to/file.crt",
                    "--pyxis-ssl-keyfile",
                    "/path/to/file.key",
                    "--manifest-digest",
                    mock.ANY,
                ],
                {},
            ),
            mock.call(
                ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-upload-signatures"),
                "pubtools-pyxis-upload-signature",
                [
                    "--pyxis-server",
                    "pyxis-url.com",
                    "--pyxis-ssl-crtfile",
                    "/path/to/file.crt",
                    "--pyxis-ssl-keyfile",
                    "/path/to/file.key",
                    "--request-threads",
                    "7",
                    "--signatures",
                    mock.ANY,
                ],
                {},
                False,
            ),
            mock.call(
                ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-get-signatures"),
                "pubtools-pyxis-get-signatures",
                [
                    "--pyxis-server",
                    "pyxis-url.com",
                    "--pyxis-ssl-crtfile",
                    "/path/to/file.crt",
                    "--pyxis-ssl-keyfile",
                    "/path/to/file.key",
                    "--manifest-digest",
                    mock.ANY,
                ],
                {},
            ),
            # TODO: Uncomment when cosign signature removal is enabled
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@manifest_list_digest",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:1111111111",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:2222222222",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:3333333333",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:5555555555",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:1111111111",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:2222222222",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:3333333333",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:5555555555",
            #     ],
            #     {},
            # ),
        ]
    )
    msg_signer_wrapper_save_signatures_file.assert_any_call(
        [
            {
                "manifest_digest": "sha256:bd6eba96070efe"
                "86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                "reference": "some-registry.com/iib-namespace/new-index-image:8",
                "repository": "iib-namespace/new-index-image",
                "sig_key_id": "sig-key",
                "signature_data": "claim1",
            }
        ]
    )
    assert mock_run_tag_images.call_count == 2
    assert mock_run_tag_images.call_args_list[0] == mock.call(
        "some-registry.com/iib-namespace/new-index-image:8",
        [
            "quay.io/some-namespace/operators----index-image:8",
        ],
        True,
        target_settings,
    )
    assert mock_run_tag_images.call_args_list[1] == mock.call(
        "some-registry.com/iib-namespace/iib:8-1",
        [
            "quay.io/some-namespace/operators----index-image:8-timestamp",
        ],
        True,
        target_settings,
    )


@mock.patch("pubtools._quay.iib_operations.ContainerImagePusher.run_tag_images")
@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_add_bundles")
@mock.patch("pubtools._quay.iib_operations.verify_target_settings")
def test_task_iib_build_from_scratch_missing_manifest_list(
    mock_verify_target_settings,
    mock_iib_add_bundles,
    mock_run_tag_images,
    mock_timestamp,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
    msg_signer_wrapper_save_signatures_file,
    signer_wrapper_remove_signatures,
    fake_quay_client_get_operator_quay_client,
    target_settings,
    fake_cert_key_paths,
    src_manifest_list,
    v2s1_manifest,
    fixture_run_in_parallel,
):
    fake_setup(
        fake_quay_client_get_operator_quay_client,
        mock_iib_add_bundles,
        signer_wrapper_entry_point,
        signer_wrapper_run_entry_point,
    )
    with requests_mock.Mocker() as m:
        m.get(
            "https://quay.io/v2/some-namespace/operators----index-image/manifests/8",
            json=None,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        iib_operations.task_iib_build_from_scratch(
            ["bundle1", "bundle2"],
            ["arch1", "arch2"],
            "8",
            ["some-key"],
            "1",
            target_settings,
        )

    mock_verify_target_settings.assert_called_once_with(target_settings)
    mock_iib_add_bundles.assert_called_once_with(
        bundles=["bundle1", "bundle2"],
        archs=["arch1", "arch2"],
        build_tags=["8-1"],
        target_settings=target_settings,
    )
    signer_wrapper_entry_point.assert_has_calls(
        [
            mock.call(
                config_file="test-config.yml",
                signing_key="some-key",
                reference=[
                    "some-registry1.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8-timestamp",
                ],
                digest=[
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                ],
                task_id="1",
            ),
            # cosign
            mock.call(
                config_file="test-config.yml",
                signing_key="some-key",
                reference=[
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                    "quay.io/some-namespace/operators----index-image:8",
                    "quay.io/some-namespace/operators----index-image:8-timestamp",
                ],
                digest=[
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                ],
                identity=[
                    "some-registry1.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8-timestamp",
                    "some-registry1.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8-timestamp",
                ],
            ),
        ]
    )
    signer_wrapper_run_entry_point.assert_has_calls(
        [
            mock.call(
                ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-upload-signatures"),
                "pubtools-pyxis-upload-signature",
                [
                    "--pyxis-server",
                    "pyxis-url.com",
                    "--pyxis-ssl-crtfile",
                    "/path/to/file.crt",
                    "--pyxis-ssl-keyfile",
                    "/path/to/file.key",
                    "--request-threads",
                    "7",
                    "--signatures",
                    "@signature_file",
                ],
                {},
                False,
            ),
        ]
    )
    msg_signer_wrapper_save_signatures_file.assert_any_call(
        [
            {
                "manifest_digest": "sha256:bd6eba96070efe"
                "86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                "reference": "some-registry.com/iib-namespace/new-index-image:8",
                "repository": "iib-namespace/new-index-image",
                "sig_key_id": "sig-key",
                "signature_data": "claim1",
            }
        ]
    )
    assert mock_run_tag_images.call_count == 2
    assert mock_run_tag_images.call_args_list[0] == mock.call(
        "some-registry.com/iib-namespace/new-index-image:8",
        [
            "quay.io/some-namespace/operators----index-image:8",
        ],
        True,
        target_settings,
    )
    assert mock_run_tag_images.call_args_list[1] == mock.call(
        "some-registry.com/iib-namespace/iib:8-1",
        [
            "quay.io/some-namespace/operators----index-image:8-timestamp",
        ],
        True,
        target_settings,
    )


@mock.patch("pubtools._quay.iib_operations.ContainerImagePusher.run_tag_images")
@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_add_bundles")
@mock.patch("pubtools._quay.iib_operations.verify_target_settings")
def test_task_iib_build_from_scratch_operator_ns(
    mock_verify_target_settings,
    mock_iib_add_bundles,
    mock_run_tag_images,
    mock_timestamp,
    mock_get_index_image_signatures,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
    msg_signer_wrapper_save_signatures_file,
    signer_wrapper_remove_signatures,
    fake_quay_client_get_operator_quay_client,
    target_settings,
    fake_cert_key_paths,
    v2s1_manifest,
    src_manifest_list,
    fixture_run_in_parallel_signer,
):
    target_settings["quay_operator_namespace"] = "operator-ns"
    fake_setup(
        fake_quay_client_get_operator_quay_client,
        mock_iib_add_bundles,
        signer_wrapper_entry_point,
        signer_wrapper_run_entry_point,
    )
    # mock_get_index_image_signatures)
    with requests_mock.Mocker() as m:
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/operator-ns/operators----index-image/manifests/8",
            src_manifest_list,
            v2s1_manifest,
        )
        # call to remove old signatures
        mock_manifest_list_requests(
            m,
            "https://quay.io"
            "/v2/some-namespace/operators----index-image/manifests/manifest_list_digest",
            src_manifest_list,
            v2s1_manifest,
        )
        iib_operations.task_iib_build_from_scratch(
            ["bundle1", "bundle2"],
            ["arch1", "arch2"],
            "8",
            ["some-key"],
            "1",
            target_settings,
        )

    mock_verify_target_settings.assert_called_once_with(target_settings)
    mock_iib_add_bundles.assert_called_once_with(
        bundles=["bundle1", "bundle2"],
        archs=["arch1", "arch2"],
        build_tags=["8-1"],
        target_settings=target_settings,
    )
    signer_wrapper_entry_point.assert_has_calls(
        [
            mock.call(
                config_file="test-config.yml",
                signing_key="some-key",
                reference=[
                    "some-registry1.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8-timestamp",
                ],
                digest=[
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                ],
                task_id="1",
            ),
            # cosign
            mock.call(
                config_file="test-config.yml",
                signing_key="some-key",
                reference=[
                    "quay.io/operator-ns/operators----index-image:8",
                    "quay.io/operator-ns/operators----index-image:8",
                    "quay.io/operator-ns/operators----index-image:8-timestamp",
                    "quay.io/operator-ns/operators----index-image:8-timestamp",
                    "quay.io/operator-ns/operators----index-image:8",
                    "quay.io/operator-ns/operators----index-image:8-timestamp",
                    "quay.io/operator-ns/operators----index-image:8",
                    "quay.io/operator-ns/operators----index-image:8-timestamp",
                ],
                digest=[
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:a1a1a1",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                    "sha256:bd6eba96070efe86b64b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                ],
                identity=[
                    "some-registry1.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8-timestamp",
                    "some-registry1.com/operators/index-image:8",
                    "some-registry1.com/operators/index-image:8-timestamp",
                    "some-registry2.com/operators/index-image:8",
                    "some-registry2.com/operators/index-image:8-timestamp",
                ],
            ),
        ]
    )
    signer_wrapper_run_entry_point.assert_has_calls(
        [
            mock.call(
                ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-get-signatures"),
                "pubtools-pyxis-get-signatures",
                [
                    "--pyxis-server",
                    "pyxis-url.com",
                    "--pyxis-ssl-crtfile",
                    "/path/to/file.crt",
                    "--pyxis-ssl-keyfile",
                    "/path/to/file.key",
                    "--manifest-digest",
                    mock.ANY,
                ],
                {},
            ),
            mock.call(
                ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-upload-signatures"),
                "pubtools-pyxis-upload-signature",
                [
                    "--pyxis-server",
                    "pyxis-url.com",
                    "--pyxis-ssl-crtfile",
                    "/path/to/file.crt",
                    "--pyxis-ssl-keyfile",
                    "/path/to/file.key",
                    "--request-threads",
                    "7",
                    "--signatures",
                    mock.ANY,
                ],
                {},
                False,
            ),
            mock.call(
                ("pubtools-pyxis", "console_scripts", "pubtools-pyxis-get-signatures"),
                "pubtools-pyxis-get-signatures",
                [
                    "--pyxis-server",
                    "pyxis-url.com",
                    "--pyxis-ssl-crtfile",
                    "/path/to/file.crt",
                    "--pyxis-ssl-keyfile",
                    "/path/to/file.key",
                    "--manifest-digest",
                    mock.ANY,
                ],
                {},
            ),
            # TODO: Uncomment when cosign signature removal is enabled
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@manifest_list_digest",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:1111111111",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:2222222222",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:3333333333",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:5555555555",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:1111111111",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:2222222222",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:3333333333",
            #     ],
            #     {},
            # ),
            # mock.call(
            #     ("pubtools-sign", "modules", "pubtools-sign-cosign-signature-list"),
            #     None,
            #     [
            #         "test-config.yml",
            #         "quay.io/some-namespace/operators----index-image@sha256:5555555555",
            #     ],
            #     {},
            # ),
        ]
    )
    msg_signer_wrapper_save_signatures_file.assert_any_call(
        [
            {
                "manifest_digest": "sha256:bd6eba96070efe86b6"
                "4b9a212680ca6d46a2e30f0a7d8e539f657eabc45c35a6",
                "reference": "some-registry.com/iib-namespace/new-index-image:8",
                "repository": "iib-namespace/new-index-image",
                "sig_key_id": "sig-key",
                "signature_data": "claim1",
            }
        ]
    )
    assert mock_run_tag_images.call_count == 2
    assert mock_run_tag_images.call_args_list[0] == mock.call(
        "some-registry.com/iib-namespace/new-index-image:8",
        [
            "quay.io/operator-ns/operators----index-image:8",
        ],
        True,
        target_settings,
    )
    assert mock_run_tag_images.call_args_list[1] == mock.call(
        "some-registry.com/iib-namespace/iib:8-1",
        [
            "quay.io/operator-ns/operators----index-image:8-timestamp",
        ],
        True,
        target_settings,
    )


@mock.patch("pubtools._quay.iib_operations.ContainerImagePusher.run_tag_images")
@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_add_deprecations")
@mock.patch("pubtools._quay.iib_operations.verify_target_settings")
def test_task_iib_add_deprecations(
    mock_verify_target_settings,
    mock_iib_add_deprecations,
    mock_run_tag_images,
    mock_timestamp,
    signer_wrapper_entry_point,
    signer_wrapper_run_entry_point,
    msg_signer_wrapper_save_signatures_file,
    signer_wrapper_remove_signatures,
    fake_quay_client_get_operator_quay_client,
    target_settings,
    fake_cert_key_paths,
    v2s1_manifest,
    src_manifest_list,
    fixture_run_in_parallel,
):
    fake_setup(
        fake_quay_client_get_operator_quay_client,
        mock_iib_add_deprecations,
        signer_wrapper_entry_point,
        signer_wrapper_run_entry_point,
    )

    with requests_mock.Mocker() as m:
        mock_manifest_list_requests(
            m,
            "https://quay.io/v2/some-namespace/operators----index-image/manifests/8",
            src_manifest_list,
            v2s1_manifest,
        )
        # manifests for removal of old signatures
        m.get(
            "https://quay.io"
            "/v2/some-namespace/operators----index-image/manifests/manifest_list_digest",
            json=src_manifest_list,
            headers={
                "Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json",
                "docker-content-digest": "manifest_list_digest",
            },
        )

        iib_operations.task_iib_add_deprecations(
            "some-registry.com/redhat-namespace/new-index-image:5",
            '{"a": "b"}',
            "operator1",
            ["some-key"],
            "1",
            target_settings,
        )

    mock_verify_target_settings.assert_called_once_with(target_settings)
    mock_iib_add_deprecations.assert_called_once_with(
        index_image="some-registry.com/redhat-namespace/new-index-image:5",
        deprecation_schema='{"a": "b"}',
        operator_package="operator1",
        build_tags=["5-1"],
        target_settings=target_settings,
    )
    assert mock_run_tag_images.call_count == 2
    assert mock_run_tag_images.call_args_list[0] == mock.call(
        "some-registry.com/iib-namespace/new-index-image:8",
        [
            "quay.io/some-namespace/operators----index-image:8",
        ],
        True,
        target_settings,
    )
    assert mock_run_tag_images.call_args_list[1] == mock.call(
        "some-registry.com/iib-namespace/iib:8-1",
        [
            "quay.io/some-namespace/operators----index-image:8-timestamp",
        ],
        True,
        target_settings,
    )
    signer_wrapper_remove_signatures.mock_calls == [
        mock.call([1]),
        mock.call([("operators/index-image", "sha256:5555555555")]),
    ]


@mock.patch("pubtools._quay.iib_operations.task_iib_add_bundles")
def test_iib_add_entrypoint(mock_add_bundles, target_settings):
    iib_operations.iib_add_entrypoint(
        ["bundle1", "bundle2"],
        ["arch1", "arch2"],
        "some-registry.com/index-image:5",
        ["bundle3", "bundle4"],
        ["some-key"],
        "1",
        target_settings,
    )

    mock_add_bundles.assert_called_once_with(
        ["bundle1", "bundle2"],
        ["arch1", "arch2"],
        "some-registry.com/index-image:5",
        ["bundle3", "bundle4"],
        ["some-key"],
        "1",
        target_settings,
    )


@mock.patch("pubtools._quay.iib_operations.task_iib_remove_operators")
def test_iib_remove_entrypoint(mock_remove_operators, target_settings):
    iib_operations.iib_remove_entrypoint(
        ["operator1", "operator2"],
        ["arch1", "arch2"],
        "some-registry.com/index-image:5",
        ["some-key"],
        "1",
        target_settings,
    )

    mock_remove_operators.assert_called_once_with(
        ["operator1", "operator2"],
        ["arch1", "arch2"],
        "some-registry.com/index-image:5",
        ["some-key"],
        "1",
        target_settings,
    )


@mock.patch("pubtools._quay.iib_operations.task_iib_build_from_scratch")
def test_iib_from_scratch_entrypoint(mock_build_from_scratch, target_settings):
    iib_operations.iib_from_scratch_entrypoint(
        ["bundle1", "bundle2"],
        ["arch1", "arch2"],
        "12",
        ["some-key"],
        "1",
        target_settings,
    )

    mock_build_from_scratch.assert_called_once_with(
        ["bundle1", "bundle2"],
        ["arch1", "arch2"],
        "12",
        ["some-key"],
        "1",
        target_settings,
    )


@mock.patch("pubtools._quay.iib_operations.task_iib_add_deprecations")
def test_iib_add_deprecations(mock_add_deprecations, target_settings):
    iib_operations.iib_add_deprecations_entrypoint(
        "some-registry.com/redhat-namespace/new-index-image:5",
        '{"a": "b"}',
        "operator1",
        ["some-key"],
        "1",
        target_settings,
    )

    mock_add_deprecations.assert_called_once_with(
        "some-registry.com/redhat-namespace/new-index-image:5",
        '{"a": "b"}',
        "operator1",
        ["some-key"],
        "1",
        target_settings,
    )


@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_add_bundles")
def test_task_iib_add_bundles_fail(mock_iib_add_bundles, target_settings):
    mock_iib_add_bundles.return_value = False
    with pytest.raises(SystemExit):
        iib_operations.task_iib_add_bundles(
            ["bundle1", "bundle2"],
            ["arch1", "arch2"],
            "some-registry.com/redhat-namespace/new-index-image:5",
            ["bundle3", "bundle4"],
            ["some-key"],
            "1",
            target_settings,
        )


@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_remove_operators")
def test_task_iib_remove_operators_fail(mock_iib_remove_operators, target_settings):
    mock_iib_remove_operators.return_value = False
    with pytest.raises(SystemExit):
        iib_operations.task_iib_remove_operators(
            ["operator1", "operator2"],
            ["arch1", "arch2"],
            "some-registry.com/redhat-namespace/new-index-image:5",
            ["some-key"],
            "1",
            target_settings,
        )


@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_add_bundles")
def test_task_iib_build_from_scratch_fail(mock_iib_add_bundles, target_settings):
    mock_iib_add_bundles.return_value = False
    with pytest.raises(SystemExit):
        iib_operations.task_iib_build_from_scratch(
            ["bundle1", "bundle2"],
            ["arch1", "arch2"],
            "12",
            ["some-key"],
            "1",
            target_settings,
        )


@mock.patch("pubtools._quay.iib_operations.OperatorPusher.iib_add_deprecations")
def test_task_iib_add_deprecations_fail(mock_iib_add_deprecations, target_settings):
    mock_iib_add_deprecations.return_value = False
    with pytest.raises(SystemExit):
        iib_operations.task_iib_add_deprecations(
            "some-registry.com/redhat-namespace/new-index-image:5",
            '{"a": "b"}',
            "operator1",
            ["some-key"],
            "1",
            target_settings,
        )



import mock
import pytest
import requests_mock

from pubtools._quay import merge_manifest_list
from .test_ml_merger import old_ml, new_ml, merged_ml


@mock.patch("pubtools._quay.merge_manifest_list.add_args_env_variables")
@mock.patch("pubtools._quay.manifest_list_merger.ManifestListMerger")
@mock.patch("pubtools._quay.manifest_list_merger.ManifestListMerger.merge_manifest_lists")
def test_arg_parser_required_args(mock_merge_manifest_lists, mock_list_merger, mock_set_env_vars):
    required_args = [
        "dummy",
        "--source-ref",
        "quay.io/repo/souce-image:1",
        "--dest-ref",
        "quay.io/repo/target-image:1",
        "--source-quay-user",
        "src-user",
        "--source-quay-password",
        "src-token",
        "--dest-quay-user",
        "dest-user",
        "--dest-quay-password",
        "dest-token",
    ]
    merge_manifest_list.merge_manifest_list_main(required_args)
    called_args, _ = mock_set_env_vars.call_args

    assert called_args[0].source_ref == "quay.io/repo/souce-image:1"
    assert called_args[0].dest_ref == "quay.io/repo/target-image:1"
    assert called_args[0].source_quay_user == "src-user"
    assert called_args[0].source_quay_password == "src-token"
    assert called_args[0].dest_quay_user == "dest-user"
    assert called_args[0].dest_quay_password == "dest-token"


@mock.patch("pubtools._quay.merge_manifest_list.add_args_env_variables")
@mock.patch("pubtools._quay.manifest_list_merger.ManifestListMerger")
@mock.patch("pubtools._quay.manifest_list_merger.ManifestListMerger.merge_manifest_lists")
def test_arg_parser_missing_required(
    mock_merge_manifest_lists, mock_list_merger, mock_set_env_vars
):
    missing_args = [
        "dummy",
        "--source-ref",
        "quay.io/repo/souce-image:1",
        "--source-quay-user",
        "src-user",
        "--source-quay-password",
        "src-token",
    ]

    with pytest.raises(SystemExit) as system_error:
        merge_manifest_list.merge_manifest_list_main(missing_args)

    assert system_error.type == SystemExit
    assert system_error.value.code == 2
    mock_merge_manifest_lists.assert_not_called()


@mock.patch("pubtools._quay.manifest_list_merger.ManifestListMerger")
@mock.patch("pubtools._quay.manifest_list_merger.ManifestListMerger.merge_manifest_lists")
def test_arg_parser_dest_digest(mock_merge_manifest_lists, mock_list_merger):
    args = [
        "dummy",
        "--source-ref",
        "quay.io/repo/souce-image:1",
        "--dest-ref",
        "quay.io/repo/target-image@sha256:dfg5dfgd6f5g6",
        "--source-quay-user",
        "src-user",
        "--source-quay-password",
        "src-token",
        "--dest-quay-user",
        "dest-user",
        "--dest-quay-password",
        "dest-token",
    ]

    with pytest.raises(ValueError, match="Destination must be specified via tag, not digest"):
        merge_manifest_list.merge_manifest_list_main(args)

    mock_merge_manifest_lists.assert_not_called()


@mock.patch("pubtools._quay.manifest_list_merger.ManifestListMerger")
@mock.patch("pubtools._quay.manifest_list_merger.ManifestListMerger.merge_manifest_lists")
def test_arg_parser_missing_password(mock_merge_manifest_lists, mock_list_merger):
    args = [
        "dummy",
        "--source-ref",
        "quay.io/repo/souce-image:1",
        "--dest-ref",
        "quay.io/repo/target-image:1",
        "--source-quay-user",
        "src-user",
        "--dest-quay-user",
        "dest-user",
    ]

    with pytest.raises(
        ValueError, match="Quay password must be set for both source and dest images"
    ):
        merge_manifest_list.merge_manifest_list_main(args)

    mock_merge_manifest_lists.assert_not_called()


def test_merge_manifest_list_full():
    args = [
        "dummy",
        "--source-ref",
        "quay.io/src/image:1",
        "--dest-ref",
        "quay.io/dest/image:1",
        "--source-quay-user",
        "src-user",
        "--source-quay-password",
        "src-token",
        "--dest-quay-user",
        "dest-user",
        "--dest-quay-password",
        "dest-token",
    ]
    with requests_mock.Mocker() as m:
        m.get(
            "https://quay.io/v2/src/image/manifests/1",
            json=new_ml,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        m.get(
            "https://quay.io/v2/dest/image/manifests/1",
            json=old_ml,
            headers={"Content-Type": "application/vnd.docker.distribution.manifest.list.v2+json"},
        )
        m.put("https://quay.io/v2/dest/image/manifests/1", status_code=200)

        merge_manifest_list.merge_manifest_list_main(args)
        assert m.call_count == 3
        sent_ml = m.request_history[-1].json()
        sent_ml["manifests"].sort(key=lambda manifest: manifest["digest"])
        merged_ml["manifests"].sort(key=lambda manifest: manifest["digest"])

        assert sent_ml == merged_ml



import sys

import mock

from pubtools._quay import tag_images

sys.modules["rhmsg"] = mock.MagicMock()
sys.modules["rhmsg.activemq"] = mock.MagicMock()
module_mock = mock.MagicMock()
sys.modules["rhmsg.activemq.producer"] = module_mock


@mock.patch("pubtools._quay.tag_images.LocalExecutor")
def test_run_tag_entrypoint_local_success(mock_local_executor, hookspy):
    args = [
        "dummy",
        "--source-ref",
        "quay.io/repo/souce-image:1",
        "--dest-ref",
        "quay.io/repo/target-image:1",
    ]
    mock_skopeo_login = mock.MagicMock()
    mock_local_executor.return_value.skopeo_login = mock_skopeo_login
    mock_local_executor.return_value.__enter__.return_value = mock_local_executor.return_value
    mock_tag_images = mock.MagicMock()
    mock_local_executor.return_value.tag_images = mock_tag_images

    tag_images.tag_images_main(args)

    mock_local_executor.assert_called_once_with()
    mock_skopeo_login.assert_called_once_with("quay.io", None, None)
    mock_tag_images.assert_called_once_with(
        "quay.io/repo/souce-image:1", ["quay.io/repo/target-image:1"], False
    )

    assert hookspy == [
        ("task_start", {}),
        (
            "quay_images_tagged",
            {
                "dest_refs": ["quay.io/repo/target-image:1"],
                "source_ref": "quay.io/repo/souce-image:1",
            },
        ),
        ("task_stop", {"failed": False}),
    ]


@mock.patch("pubtools._quay.tag_images.LocalExecutor")
def test_run_tag_entrypoint_local_success_all_arch(mock_local_executor):
    args = [
        "dummy",
        "--source-ref",
        "src.quay.io/repo/souce-image:1",
        "--dest-ref",
        "quay.io/repo/target-image:1",
        "--all-arch",
        "--source-quay-host",
        "src.quay.io",
        "--source-quay-user",
        "src-user",
        "--source-quay-password",
        "src-password",
    ]
    mock_skopeo_login = mock.MagicMock()
    mock_local_executor.return_value.skopeo_login = mock_skopeo_login
    mock_local_executor.return_value.__enter__.return_value = mock_local_executor.return_value
    mock_tag_images = mock.MagicMock()
    mock_local_executor.return_value.tag_images = mock_tag_images

    tag_images.tag_images_main(args)

    mock_local_executor.assert_called_once_with()
    assert len(mock_skopeo_login.mock_calls) == 2
    assert mock_skopeo_login.call_args_list == [
        mock.call("quay.io", None, None),
        mock.call("src.quay.io", "src-user", "src-password"),
    ]
    mock_tag_images.assert_called_once_with(
        "src.quay.io/repo/souce-image:1", ["quay.io/repo/target-image:1"], True
    )


@mock.patch("pubtools._quay.tag_images.RemoteExecutor")
def test_run_tag_entrypoint_remote_success(mock_remote_executor):
    args = [
        "dummy",
        "--source-ref",
        "quay.io/repo/souce-image:1",
        "--dest-ref",
        "quay.io/repo/target-image:1",
        "--remote-exec",
        "--ssh-remote-host",
        "127.0.0.1",
        "--ssh-reject-unknown-host",
        "--ssh-username",
        "dummy",
        "--ssh-password",
        "123456",
        "--ssh-key-filename",
        "/path/to/file.key",
    ]
    mock_skopeo_login = mock.MagicMock()
    mock_remote_executor.return_value.skopeo_login = mock_skopeo_login
    mock_remote_executor.return_value.__enter__.return_value = mock_remote_executor.return_value
    mock_tag_images = mock.MagicMock()
    mock_remote_executor.return_value.tag_images = mock_tag_images

    tag_images.tag_images_main(args)

    mock_remote_executor.assert_called_once_with(
        "127.0.0.1", "dummy", "/path/to/file.key", "123456", None, False
    )
    mock_skopeo_login.assert_called_once_with("quay.io", None, None)
    mock_tag_images.assert_called_once_with(
        "quay.io/repo/souce-image:1", ["quay.io/repo/target-image:1"], False
    )


@mock.patch("pubtools._quay.tag_images.ContainerExecutor")
def test_run_tag_entrypoint_container_success(mock_container_executor):
    args = [
        "dummy",
        "--source-ref",
        "quay.io/repo/souce-image:1",
        "--dest-ref",
        "quay.io/repo/target-image:1",
        "--container-exec",
        "--container-image",
        "quay.io/some/image:1",
        "--docker-url",
        "some-url.com",
        "--docker-timeout",
        "120",
        "--docker-verify-tls",
        "--docker-cert-path",
        "/some/path",
        "--registry-username",
        "registry-user",
        "--registry-password",
        "registry-passwd",
    ]
    mock_skopeo_login = mock.MagicMock()
    mock_container_executor.return_value.skopeo_login = mock_skopeo_login
    mock_container_executor.return_value.__enter__.return_value = (
        mock_container_executor.return_value
    )
    mock_tag_images = mock.MagicMock()
    mock_container_executor.return_value.tag_images = mock_tag_images

    tag_images.tag_images_main(args)

    mock_container_executor.assert_called_once_with(
        "quay.io/some/image:1",
        "some-url.com",
        120,
        True,
        "/some/path",
        "registry-user",
        "registry-passwd",
    )
    mock_skopeo_login.assert_called_once_with("quay.io", None, None)
    mock_tag_images.assert_called_once_with(
        "quay.io/repo/souce-image:1", ["quay.io/repo/target-image:1"], False
    )
